@@ -3,6 +3,7 @@
     Copyright (C) 1998-2005 Ivo Timmermans,
                   2000-2010 Guus Sliepen <guus@tinc-vpn.org>
                   2006      Scott Lamb <slamb@slamb.org>
+                  2010      Brandon Black <blblack@gmail.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -149,7 +150,7 @@ bool read_rsa_private_key(void) {
 	struct stat s;
 
 	if(get_config_string(lookup_config(config_tree, "PrivateKey"), &key)) {
-		if(!get_config_string(lookup_config(myself->connection->config_tree, "PublicKey"), &pubkey)) {
+		if(!get_config_string(lookup_config(config_tree, "PublicKey"), &pubkey)) {
 			logger(LOG_ERR, "PrivateKey used but no PublicKey found!");
 			return false;
 		}
@@ -270,15 +271,16 @@ bool setup_myself(void) {
 	config_t *cfg;
 	subnet_t *subnet;
 	char *name, *hostname, *mode, *afname, *cipher, *digest;
+	char *fname = NULL;
 	char *address = NULL;
 	char *envp[5];
 	struct addrinfo *ai, *aip, hint = {0};
 	bool choice;
 	int i, err;
+	int replaywin_int;
 
 	myself = new_node();
 	myself->connection = new_connection();
-	init_configuration(&myself->connection->config_tree);
 
 	myself->hostname = xstrdup("MYSELF");
 	myself->connection->hostname = xstrdup("MYSELF");
@@ -299,17 +301,15 @@ bool setup_myself(void) {
 
 	myself->name = name;
 	myself->connection->name = xstrdup(name);
-
-	if(!read_connection_config(myself->connection)) {
-		logger(LOG_ERR, "Cannot open host configuration file for myself!");
-		return false;
-	}
+	xasprintf(&fname, "%s/hosts/%s", confbase, name);
+	read_config_options(config_tree, name);
+	read_config_file(config_tree, fname);
+	free(fname);
 
 	if(!read_rsa_private_key())
 		return false;
 
-	if(!get_config_string(lookup_config(config_tree, "Port"), &myport)
-			&& !get_config_string(lookup_config(myself->connection->config_tree, "Port"), &myport))
+	if(!get_config_string(lookup_config(config_tree, "Port"), &myport))
 		myport = xstrdup("655");
 
 	if(!atoi(myport)) {
@@ -324,7 +324,7 @@ bool setup_myself(void) {
 
 	/* Read in all the subnets specified in the host configuration file */
 
-	cfg = lookup_config(myself->connection->config_tree, "Subnet");
+	cfg = lookup_config(config_tree, "Subnet");
 
 	while(cfg) {
 		if(!get_config_subnet(cfg, &subnet))
@@ -332,7 +332,7 @@ bool setup_myself(void) {
 
 		subnet_add(myself, subnet);
 
-		cfg = lookup_config_next(myself->connection->config_tree, cfg);
+		cfg = lookup_config_next(config_tree, cfg);
 	}
 
 	/* Check some options */
@@ -343,12 +343,6 @@ bool setup_myself(void) {
 	if(get_config_bool(lookup_config(config_tree, "TCPOnly"), &choice) && choice)
 		myself->options |= OPTION_TCPONLY;
 
-	if(get_config_bool(lookup_config(myself->connection->config_tree, "IndirectData"), &choice) && choice)
-		myself->options |= OPTION_INDIRECT;
-
-	if(get_config_bool(lookup_config(myself->connection->config_tree, "TCPOnly"), &choice) && choice)
-		myself->options |= OPTION_TCPONLY;
-
 	if(myself->options & OPTION_TCPONLY)
 		myself->options |= OPTION_INDIRECT;
 
@@ -386,14 +380,12 @@ bool setup_myself(void) {
 	}
 
 	choice = true;
-	get_config_bool(lookup_config(myself->connection->config_tree, "PMTUDiscovery"), &choice);
 	get_config_bool(lookup_config(config_tree, "PMTUDiscovery"), &choice);
 	if(choice)
 		myself->options |= OPTION_PMTU_DISCOVERY;
 
 	choice = true;
 	get_config_bool(lookup_config(config_tree, "ClampMSS"), &choice);
-	get_config_bool(lookup_config(myself->connection->config_tree, "ClampMSS"), &choice);
 	if(choice)
 		myself->options |= OPTION_CLAMP_MSS;
 
@@ -415,6 +407,28 @@ bool setup_myself(void) {
 	} else
 		maxtimeout = 900;
 
+	if(get_config_int(lookup_config(config_tree, "UDPRcvBuf"), &udp_rcvbuf)) {
+		if(udp_rcvbuf <= 0) {
+			logger(LOG_ERR, "UDPRcvBuf cannot be negative!");
+			return false;
+		}
+	}
+
+	if(get_config_int(lookup_config(config_tree, "UDPSndBuf"), &udp_sndbuf)) {
+		if(udp_sndbuf <= 0) {
+			logger(LOG_ERR, "UDPSndBuf cannot be negative!");
+			return false;
+		}
+	}
+
+	if(get_config_int(lookup_config(config_tree, "ReplayWindow"), &replaywin_int)) {
+		if(replaywin_int < 0) {
+			logger(LOG_ERR, "ReplayWindow cannot be negative!");
+			return false;
+		}
+		replaywin = (unsigned)replaywin_int;
+	}
+
 	if(get_config_string(lookup_config(config_tree, "AddressFamily"), &afname)) {
 		if(!strcasecmp(afname, "IPv4"))
 			addressfamily = AF_INET;
@@ -434,7 +448,7 @@ bool setup_myself(void) {
 	/* Generate packet encryption key */
 
 	if(get_config_string
-	   (lookup_config(myself->connection->config_tree, "Cipher"), &cipher)) {
+	   (lookup_config(config_tree, "Cipher"), &cipher)) {
 		if(!strcasecmp(cipher, "none")) {
 			myself->incipher = NULL;
 		} else {
@@ -462,7 +476,7 @@ bool setup_myself(void) {
 	
 	/* Check if we want to use message authentication codes... */
 
-	if(get_config_string(lookup_config(myself->connection->config_tree, "Digest"), &digest)) {
+	if(get_config_string(lookup_config(config_tree, "Digest"), &digest)) {
 		if(!strcasecmp(digest, "none")) {
 			myself->indigest = NULL;
 		} else {
@@ -478,7 +492,7 @@ bool setup_myself(void) {
 
 	myself->connection->outdigest = EVP_sha1();
 
-	if(get_config_int(lookup_config(myself->connection->config_tree, "MACLength"), &myself->inmaclength)) {
+	if(get_config_int(lookup_config(config_tree, "MACLength"), &myself->inmaclength)) {
 		if(myself->indigest) {
 			if(myself->inmaclength > myself->indigest->md_size) {
 				logger(LOG_ERR, "MAC length exceeds size of digest!");
@@ -495,7 +509,7 @@ bool setup_myself(void) {
 
 	/* Compression */
 
-	if(get_config_int(lookup_config(myself->connection->config_tree, "Compression"), &myself->incompression)) {
+	if(get_config_int(lookup_config(config_tree, "Compression"), &myself->incompression)) {
 		if(myself->incompression < 0 || myself->incompression > 11) {
 			logger(LOG_ERR, "Bogus compression level!");
 			return false;
